@@ -9,19 +9,31 @@ function(method, model, k, control=fitsaemodel.control(...), ...){
    eps <- .Machine$double.eps
    #switch method
    if (method == "ml"){
-      k <- control$maxk
-      kappa <- 1
+      k <- rep(control$maxk, 3)
+      k.report <- k[1]
+      kappa <- rep(1.0, 2)
       methodName <- list(type="Maximum likelihood estimation")
    }else{
       #check whether k exists 
       if (missing(k)) stop("Robustness tuning constant is missing! \n")
-      #check whether the range of k makes sense
-      if (k < 0) stop("Robustness tuning constant k must be > 0.\n")
-      kmin <- eps^(1/4) 
-      if (k < kmin) stop(paste("Robustness tuning constant k is too small (note kmin = ", kmin, ").\n"))
-      #consistency correction term for Huber-Proposal-2 scale estimate
-      kappa <- .computekappa(k)
-      methodName <- list(type="Huber-type M-estimation", tuning=list(k=k))
+      if (is.list(k)){
+	 if( any(is.na(match(names(k), c("beta", "v", "d")))) ) stop("List of robustness tuning constants 'k' must consist\nof the elements 'beta', 'v', and 'd'!\n")
+	 k <- c(k$beta, k$v, k$d)
+	 if ( any(k <= 0) ) stop("Robustness tuning constants must be larger than zero!\n")
+	 kappa <- c(.computekappa(k[2]), .computekappa(k[3]))
+	 k.report <- k
+      }else{
+	 if ( length(k) != 1 ) stop("Robustness tuning constant 'k' must be either\na scalar or a list!\n")
+	 if (k < 0) stop("Robustness tuning constant k must be > 0!\n")
+	 kmin <- eps^(1/4)
+	 if (k < kmin) stop(paste("Robustness tuning constant k is too small (note kmin = ", kmin, ").\n"))	 
+	 #consistency correction term for Huber-Proposal-2 scale estimate   
+	 k <- rep(k, 3)
+	 k.report <- k[1]
+	 kappa <- .computekappa(k)
+	 kappa <- rep(kappa, 2)
+      }
+      methodName <- list(type="Huber-type M-estimation", tuning=list(k=k.report))
    } 
    #data preparation
    x <- model$X
@@ -43,6 +55,10 @@ function(method, model, k, control=fitsaemodel.control(...), ...){
    #acc specification (note that we take abs() to ensure that it is not negative)
    allacc <- control$acc[1]
    acc <- control$acc[2:4]
+   #type of decomposition for matrix square root
+   dec <- control$dec
+   # type of decorrelation
+   decorr <- control$decorr
    #sum of Huber downweighting
    sumwgt <- c(0, 0, 0)
    #initialize the full parameter vector
@@ -57,7 +73,7 @@ function(method, model, k, control=fitsaemodel.control(...), ...){
    #define epsd (minimal d that is different from zero)
    epsd <- eps^(1/4)
    #call
-   tmp <- .Fortran("drsaehub", n=as.integer(n), p=as.integer(p), g=as.integer(g), niter=as.integer(niter), nsize=as.integer(nsize), iter=as.integer(iter), iterrecord=as.matrix(iterrecord), allacc=as.double(allacc), acc=as.matrix(acc), sumwgt=as.matrix(sumwgt), xmat=as.matrix(x), yvec=as.matrix(y), k=as.double(k), kappa=as.double(kappa), epsd=as.double(epsd), tau=as.matrix(tau), taurecord=as.matrix(taurecord), converged=as.integer(0))
+   tmp <- .Fortran("drsaehub", n=as.integer(n), p=as.integer(p), g=as.integer(g), niter=as.integer(niter), nsize=as.integer(nsize), iter=as.integer(iter), iterrecord=as.matrix(iterrecord), allacc=as.double(allacc), acc=as.matrix(acc), sumwgt=as.matrix(sumwgt), xmat=as.matrix(x), yvec=as.matrix(y), k=as.matrix(k), kappa=as.matrix(kappa), epsd=as.double(epsd), tau=as.matrix(tau), taurecord=as.matrix(taurecord), converged=as.integer(0), dec=as.integer(dec), decorr=as.integer(decorr))
    #return values
 #HOTFIX!, check for cycling an choose the parameter-vector estimate whose estimate of v is closer to the (robust) init (i.e., either the "lts" or "s" estimate) value. This method is not supported for init=default or ml
    converged <- tmp$converged
@@ -82,7 +98,7 @@ function(method, model, k, control=fitsaemodel.control(...), ...){
    #compute the covariance matrix of the fixed effects
    if (converged == 1){
       vcovbeta <- matrix(0, p, p)
-      o = .Fortran("drsaehubvariance", n=as.integer(n), p=as.integer(p), g=as.integer(g), nsize=as.integer(nsize), kappa=as.double(kappa), v=as.double(tau[p+1]), d=as.double(tau[p+2]), xmat=as.matrix(x), vcovbeta=as.matrix(vcovbeta))
+      o = .Fortran("drsaehubvariance", n=as.integer(n), p=as.integer(p), g=as.integer(g), nsize=as.integer(nsize), kappa=as.double(kappa), v=as.double(tau[p+1]), d=as.double(tau[p+2]), xmat=as.matrix(x), vcovbeta=as.matrix(vcovbeta), dec=as.integer(dec))
       vcovbeta <- o$vcovbeta
    }else{
       vcovbeta <- NULL
@@ -97,6 +113,7 @@ function(method, model, k, control=fitsaemodel.control(...), ...){
    attr(res, "init") <- init
    attr(res, "method") <- methodName 
    attr(res, "saemodel") <- model
+   attr(res, "dec") <- dec
    class(res) <- "fitsaemodel" 
    return(res)  
 }
@@ -106,15 +123,43 @@ function(model, init, ...){
    n <- model$n
    p <- model$p
    intercept <- model$intercept
-   # default
+   #-------------
+   # default (i.e., robust fixed-effects estimator; see AJS2011)
    if (init == 0){
-      result <- rep(1, (p+2))
+      # by default
+      k <- 1.345
+      # retrieve all the model characteristics
+      y <- model$y
+      X <- as.data.frame(model$X)
+      g <- model$g
+      areaID <- model$areaID
+      # center y by the area-specific median of y
+      y.list <- split(y, areaID)
+      y.centered.list <- lapply(y.list, function(u) u - median(u))   
+      y.centered <- unsplit(y.centered.list, areaID)
+      # center X by the area-specific mean of x
+      X.list <- split(X, areaID)
+      X.centered.list <- lapply(X.list, function(u) as.data.frame(sweep(as.matrix(u), MARGIN=2, STATS=colMeans(u))))
+      X.centered <- unsplit(X.centered.list, areaID)
+      if (intercept == 1){
+	 X.centered <- X.centered[, -1]
+	 p <- p - 1
+      }
+      # prepare the model.frame
+      mm <- model.matrix(~ -1 + as.factor(areaID))
+      x <- cbind(X.centered, mm)
+      # compute the robust fixed-effects estimator
+      initbeta <- rep(1, (p + g))
+      tmp <- .Fortran("drlm", n=as.integer(n), p=as.integer(p+g), xmat=as.matrix(x), yvec=as.matrix(y.centered), k=as.double(k), beta=as.matrix(initbeta), s=as.double(1.2), info=as.integer(1), niter=as.integer(20), acc=as.double(0.00001))
+      result <- c(0, tmp$beta[1:p], tmp$s^2, 100)
    }
-   # check wheter robustbase must be loaded
+   #-------------
+   # check whether robustbase must be loaded
    if (init > 0){
       checkrobustbase <- require(robustbase)
-      if(!checkrobustbase) stop("You cannot use either 'lts' or 's', because the \npackage 'robustbase' is not installed! \n")
+      if(!checkrobustbase) stop("You cannot use 'lts' or 's', because the \n 'robustbase' package is not installed! \n")
    }
+   #-------------
    # lts
    if (init == 1){
       x <- as.matrix(model$X)
@@ -133,6 +178,7 @@ function(model, init, ...){
       # repare return value (beta, v, d)
       result <- as.numeric(c(tmp$coefficients, tmp$raw.scale^2, 1))
    }
+   #-------------
    # lmrob.S
    if (init == 2){
       x <- as.matrix(model$X)
@@ -173,6 +219,8 @@ function(fit, reps, areameans, fixeff){
       tmp <- fitsaemodel("ml", model)
       #predict 
       predicts[j, ] <- t(robpredict(tmp, areameans, k=20000, reps=NULL)$means) - t(predrf)
+      #status bar
+      ttmp <- .C("statusbar", as.integer(j), as.integer(reps))
    }
    res <- colMeans(predicts^2)
    return(res)
